import 'dart:async';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';

/// GestureRecognizer
class IDKitGestureRecognizer extends StatefulWidget {
  const IDKitGestureRecognizer({
    super.key,
    this.child,
    this.onTap,
    this.onStart,
    this.onUpdate,
    this.onEnd,
    this.fingerCount,
    this.forcePress,
  });

  /// Callback for click event.
  final Function(SerialTapUpDetails)? onTap;

  /// Gesture zoom, move start method call.
  final Function(ScaleStartDetails)? onStart;

  /// Gesture zoom, mobile update method call.
  final Function(ScaleUpdateDetails)? onUpdate;

  /// Gesture zooming and moving end method calls.
  final Function(ScaleEndDetails)? onEnd;

  /// Callback for counting the number of fingers on the screen, one does not count.
  final Function(int)? fingerCount;

  /// The back method of on-screen finger pressure monitoring.
  final Function(ForcePressDetails)? forcePress;

  /// Subwidgets of the Gestures widget.
  final Widget? child;

  @override
  State<IDKitGestureRecognizer> createState() => _IDKitGestureRecognizerState();
}

class _IDKitGestureRecognizerState extends State<IDKitGestureRecognizer> {
  /// Record of fingers
  int count = 0;

  /// Whether to trigger the record of the finger counting end method
  bool isOnSerialTapUp = false;

  /// Whether to trigger the recording of the zoom start method
  bool isOnStart = false;

  /// Whether to trigger the recording of zoom end method
  bool isOnEnd = false;

  @override
  Widget build(BuildContext context) {
    return RawGestureDetector(
      gestures: {
        TapGestureRecognizer:
            GestureRecognizerFactoryWithHandlers<TapGestureRecognizer>(
          () => TapGestureRecognizer(),
          (instance) {
            instance.onTap = () {};
          },
        ),
        ForcePressGestureRecognizer:
            GestureRecognizerFactoryWithHandlers<ForcePressGestureRecognizer>(
          () => ForcePressGestureRecognizer(),
          (ForcePressGestureRecognizer instance) {
            instance.onStart = widget.forcePress;
            instance.onUpdate = widget.forcePress;
            instance.onEnd = widget.forcePress;
          },
        ),
        ScaleGestureRecognizer:
            GestureRecognizerFactoryWithHandlers<ScaleGestureRecognizer>(
          () => ScaleGestureRecognizer(),
          (instance) {
            instance.onStart = (ScaleStartDetails details) {
              if (!isOnSerialTapUp) count = 0;
              if (isOnEnd) return;
              if (!isOnStart) isOnStart = true;
              widget.onStart?.call(details);
            };
            instance.onUpdate = (ScaleUpdateDetails details) {
              if (isOnEnd) return;
              widget.onUpdate?.call(details);
            };
            instance.onEnd = (ScaleEndDetails details) {
              if (isOnEnd) return;
              isOnEnd = true;
              Timer.periodic(const Duration(milliseconds: 300), (timer) {
                isOnEnd = false;
                timer.cancel();
              });
              if (isOnStart) isOnStart = false;
              widget.onEnd?.call(details);
            };
          },
        ),
        SerialTapGestureRecognizer:
            GestureRecognizerFactoryWithHandlers<SerialTapGestureRecognizer>(
          () => SerialTapGestureRecognizer(),
          (SerialTapGestureRecognizer instance) {
            instance.onSerialTapDown = (details) {
              isOnSerialTapUp = false;
              count++;
            };
            instance.onSerialTapUp = (SerialTapUpDetails details) {
              isOnSerialTapUp = true;
              if (count > 1) {
                widget.fingerCount?.call(count);
              } else {
                widget.onTap?.call(details);
              }
              count = 0;
            };
          },
        ),
      },
      child: widget.child,
    );
  }
}
